/* libnih
 *
 * com.netsplit.Nih.Test_proxy.c - Auto-generated d-bus bindings
 *
 * Copyright © 2008 Scott James Remnant <scott@netsplit.com>.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif /* HAVE_CONFIG_H */


#include <dbus/dbus.h>

#include <errno.h>
#include <limits.h>

#include <nih/macros.h>
#include <nih/alloc.h>
#include <nih/string.h>
#include <nih/logging.h>
#include <nih/error.h>
#include <nih/errors.h>
#include <nih/dbus.h>

#include "com.netsplit.Nih.Test_proxy.h"


int
proxy_test_method (NihDBusProxy  *proxy,
                   const char    *input,
                   int32_t        flags,
                   char         **output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *input_value;
	char            *output_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "TestMethod");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	input_value = input;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &input_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT32, &flags)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &output_value);

	*output = nih_strdup (proxy, output_value);
	if (! *output) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_test_async_method (NihDBusProxy  *proxy,
                         const char    *input,
                         int32_t        flags,
                         char         **output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *input_value;
	char            *output_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "TestAsyncMethod");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	input_value = input;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &input_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT32, &flags)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &output_value);

	*output = nih_strdup (proxy, output_value);
	if (! *output) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_byte_to_str (NihDBusProxy  *proxy,
                   uint8_t        input,
                   char         **output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	char            *output_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "ByteToStr");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_BYTE, &input)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &output_value);

	*output = nih_strdup (proxy, output_value);
	if (! *output) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_str_to_byte (NihDBusProxy *proxy,
                   const char   *input,
                   uint8_t      *output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *input_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "StrToByte");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	input_value = input;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &input_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_BYTE) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &*output);

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_boolean_to_str (NihDBusProxy  *proxy,
                      int            input,
                      char         **output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	char            *output_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "BooleanToStr");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &input)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &output_value);

	*output = nih_strdup (proxy, output_value);
	if (! *output) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_str_to_boolean (NihDBusProxy *proxy,
                      const char   *input,
                      int          *output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *input_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "StrToBoolean");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	input_value = input;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &input_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_BOOLEAN) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &*output);

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_int16_to_str (NihDBusProxy  *proxy,
                    int16_t        input,
                    char         **output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	char            *output_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "Int16ToStr");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT16, &input)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &output_value);

	*output = nih_strdup (proxy, output_value);
	if (! *output) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_str_to_int16 (NihDBusProxy *proxy,
                    const char   *input,
                    int16_t      *output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *input_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "StrToInt16");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	input_value = input;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &input_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INT16) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &*output);

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_uint16_to_str (NihDBusProxy  *proxy,
                     uint16_t       input,
                     char         **output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	char            *output_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "UInt16ToStr");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT16, &input)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &output_value);

	*output = nih_strdup (proxy, output_value);
	if (! *output) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_str_to_uint16 (NihDBusProxy *proxy,
                     const char   *input,
                     uint16_t     *output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *input_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "StrToUInt16");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	input_value = input;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &input_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_UINT16) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &*output);

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_int32_to_str (NihDBusProxy  *proxy,
                    int32_t        input,
                    char         **output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	char            *output_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "Int32ToStr");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT32, &input)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &output_value);

	*output = nih_strdup (proxy, output_value);
	if (! *output) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_str_to_int32 (NihDBusProxy *proxy,
                    const char   *input,
                    int32_t      *output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *input_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "StrToInt32");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	input_value = input;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &input_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INT32) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &*output);

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_uint32_to_str (NihDBusProxy  *proxy,
                     uint32_t       input,
                     char         **output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	char            *output_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "UInt32ToStr");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &input)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &output_value);

	*output = nih_strdup (proxy, output_value);
	if (! *output) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_str_to_uint32 (NihDBusProxy *proxy,
                     const char   *input,
                     uint32_t     *output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *input_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "StrToUInt32");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	input_value = input;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &input_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_UINT32) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &*output);

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_int64_to_str (NihDBusProxy  *proxy,
                    int64_t        input,
                    char         **output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	char            *output_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "Int64ToStr");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT64, &input)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &output_value);

	*output = nih_strdup (proxy, output_value);
	if (! *output) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_str_to_int64 (NihDBusProxy *proxy,
                    const char   *input,
                    int64_t      *output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *input_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "StrToInt64");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	input_value = input;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &input_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INT64) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &*output);

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_uint64_to_str (NihDBusProxy  *proxy,
                     uint64_t       input,
                     char         **output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	char            *output_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "UInt64ToStr");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT64, &input)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &output_value);

	*output = nih_strdup (proxy, output_value);
	if (! *output) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_str_to_uint64 (NihDBusProxy *proxy,
                     const char   *input,
                     uint64_t     *output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *input_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "StrToUInt64");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	input_value = input;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &input_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_UINT64) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &*output);

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_double_to_str (NihDBusProxy  *proxy,
                     double         input,
                     char         **output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	char            *output_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "DoubleToStr");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_DOUBLE, &input)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &output_value);

	*output = nih_strdup (proxy, output_value);
	if (! *output) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_str_to_double (NihDBusProxy *proxy,
                     const char   *input,
                     double       *output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *input_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "StrToDouble");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	input_value = input;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &input_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_DOUBLE) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &*output);

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_object_path_to_str (NihDBusProxy  *proxy,
                          const char    *input,
                          char         **output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *input_value;
	char            *output_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "ObjectPathToStr");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	input_value = input;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_OBJECT_PATH, &input_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &output_value);

	*output = nih_strdup (proxy, output_value);
	if (! *output) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_str_to_object_path (NihDBusProxy  *proxy,
                          const char    *input,
                          char         **output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *input_value;
	char            *output_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "StrToObjectPath");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	input_value = input;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &input_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_OBJECT_PATH) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &output_value);

	*output = nih_strdup (proxy, output_value);
	if (! *output) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_signature_to_str (NihDBusProxy  *proxy,
                        const char    *input,
                        char         **output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *input_value;
	char            *output_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "SignatureToStr");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	input_value = input;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_SIGNATURE, &input_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &output_value);

	*output = nih_strdup (proxy, output_value);
	if (! *output) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_str_to_signature (NihDBusProxy  *proxy,
                        const char    *input,
                        char         **output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *input_value;
	char            *output_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "StrToSignature");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	input_value = input;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &input_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_SIGNATURE) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &output_value);

	*output = nih_strdup (proxy, output_value);
	if (! *output) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_int32_array_to_str (NihDBusProxy   *proxy,
                          const int32_t  *input,
                          size_t          input_len,
                          char          **output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	DBusMessageIter  input_iter;
	char            *output_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "Int32ArrayToStr");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	if (! dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "i", &input_iter)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	for (const int32_t *input_p = input; input_p < input + input_len; input_p++) {
		int32_t input_elem;

		input_elem = *input_p;

		if (! dbus_message_iter_append_basic (&input_iter, DBUS_TYPE_INT32, &input_elem)) {
			dbus_message_unref (message);
			nih_return_no_memory_error (-1);

		}
	}

	if (! dbus_message_iter_close_container (&iter, &input_iter)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &output_value);

	*output = nih_strdup (proxy, output_value);
	if (! *output) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_str_to_int32_array (NihDBusProxy  *proxy,
                          const char    *input,
                          int32_t      **output,
                          size_t        *output_len)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *input_value;
	DBusMessageIter  output_iter;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "StrToInt32Array");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	input_value = input;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &input_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_ARRAY) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	if (dbus_message_iter_get_element_type (&iter) != DBUS_TYPE_INT32) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_recurse (&iter, &output_iter);

	*output = NULL;
	*output_len = 0;

	while (dbus_message_iter_get_arg_type (&output_iter) != DBUS_TYPE_INVALID) {
		int32_t output_elem;

		if (dbus_message_iter_get_arg_type (&output_iter) != DBUS_TYPE_INT32) {
			dbus_message_unref (reply);
			nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

		}

		dbus_message_iter_get_basic (&output_iter, &output_elem);

		dbus_message_iter_next (&output_iter);

		*output = nih_realloc (*output, proxy, sizeof (int32_t) * ((*output_len) + 1));
		if (! *output) {
			dbus_message_unref (reply);
			nih_return_no_memory_error (-1);

		}

		(*output)[(*output_len)++] = output_elem;

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_str_array_to_str (NihDBusProxy  *proxy,
                        char * const  *input,
                        char         **output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	DBusMessageIter  input_iter;
	size_t           input_len;
	char            *output_value;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "StrArrayToStr");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	if (! dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "s", &input_iter)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	input_len = 0;
	for (char * const *input_p = input; input_p && *input_p; input_p++) {
		char *input_elem;
		char *input_elem_value;

		input_elem = *input_p;

		input_elem_value = input_elem;
		if (! dbus_message_iter_append_basic (&input_iter, DBUS_TYPE_STRING, &input_elem_value)) {
			dbus_message_unref (message);
			nih_return_no_memory_error (-1);

		}

		(input_len)++;
	}

	if (! dbus_message_iter_close_container (&iter, &input_iter)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_get_basic (&iter, &output_value);

	*output = nih_strdup (proxy, output_value);
	if (! *output) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	dbus_message_iter_next (&iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_str_to_str_array (NihDBusProxy   *proxy,
                        const char     *input,
                        char         ***output)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;
	const char      *input_value;
	DBusMessageIter  output_iter;
	size_t           output_len;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Test", "StrToStrArray");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	input_value = input;
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &input_value)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_ARRAY) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	if (dbus_message_iter_get_element_type (&iter) != DBUS_TYPE_STRING) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_iter_recurse (&iter, &output_iter);

	*output = NULL;
	output_len = 0;

	while (dbus_message_iter_get_arg_type (&output_iter) != DBUS_TYPE_INVALID) {
		char *output_elem;
		char *output_elem_value;

		if (dbus_message_iter_get_arg_type (&output_iter) != DBUS_TYPE_STRING) {
			dbus_message_unref (reply);
			nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

		}

		dbus_message_iter_get_basic (&output_iter, &output_elem_value);

		output_elem = nih_strdup (proxy, output_elem_value);
		if (! output_elem) {
			dbus_message_unref (reply);
			nih_return_no_memory_error (-1);

		}

		dbus_message_iter_next (&output_iter);

		*output = nih_realloc (*output, proxy, sizeof (char *) * ((output_len) + 1));
		if (! *output) {
			dbus_message_unref (reply);
			nih_return_no_memory_error (-1);

		}

		(*output)[(output_len)++] = output_elem;

	}

	dbus_message_iter_next (&iter);

	*output = nih_realloc (*output, proxy, sizeof (char *) * ((output_len) + 1));
	if (! *output) {
		dbus_message_unref (reply);
		nih_return_no_memory_error (-1);

	}

	(*output)[(output_len)] = NULL;

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

int
proxy_emit_signal (NihDBusProxy *proxy,
                   int32_t       signum)
{
	DBusMessage     *message;
	DBusMessageIter  iter;
	DBusMessage     *reply = NULL;
	DBusError        error;

	nih_assert (proxy != NULL);

	message = dbus_message_new_method_call (proxy->name, proxy->path, "com.netsplit.Nih.Glue", "EmitSignal");
	if (! message)
		nih_return_no_memory_error (-1);

	/* Iterate the arguments to the function and dispatch into
	 * message arguments.
	 */
	dbus_message_iter_init_append (message, &iter);

	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT32, &signum)) {
		dbus_message_unref (message);
		nih_return_no_memory_error (-1);

	}

	dbus_error_init (&error);

	/* Send the reply, appending it to the outgoing queue and blocking. */
	reply = dbus_connection_send_with_reply_and_block (proxy->conn, message, -1, &error);
	if (! reply) {
		dbus_message_unref (message);

		if (dbus_error_has_name (&error, DBUS_ERROR_NO_MEMORY)) {
			dbus_error_free (&error);
			nih_return_no_memory_error (-1);
		} else {
			nih_dbus_error_raise (error.name, error.message);
			dbus_error_free (&error);
			return -1;
		}
	}

	dbus_message_unref (message);

	/* Iterate the arguments to the reply and marshal into output
	 * arguments from our own function call.
	 */
	dbus_message_iter_init (reply, &iter);

	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INVALID) {
		dbus_message_unref (reply);
		nih_return_error (-1, NIH_DBUS_INVALID_ARGS, NIH_DBUS_INVALID_ARGS_STR);

	}

	dbus_message_unref (reply);

	return 0;
}

